library(tidyverse)

d <- read.csv('contracts.csv')

d$vendor_name <- str_to_upper(str_squish(d$vendor_name))
d$vendor_postal_code <- str_to_upper(str_squish(d$vendor_postal_code))
d$economic_object_code <- str_to_upper(str_squish(d$economic_object_code))
d$agreement_type_code <- str_to_upper(str_squish(d$agreement_type_code))
d$trade_agreement <- str_to_upper(str_squish(d$trade_agreement))
d$commodity_type <- str_to_upper(str_squish(d$commodity_type))
d$commodity_code <- str_to_upper(str_squish(d$commodity_code))
d$country_of_vendor <- str_to_upper(str_squish(d$country_of_vendor))
d$solicitation_procedure <- str_to_upper(str_squish(d$solicitation_procedure))
d$commodity_type <- str_to_upper(str_squish(d$commodity_type))
d$limited_tendering_reason <- str_to_upper(str_squish(d$limited_tendering_reason))
d$trade_agreement_exceptions <- str_to_upper(str_squish(d$trade_agreement_exceptions))
d$article_6_exceptions <- str_to_upper(str_squish(d$article_6_exceptions))
d$award_criteria <- str_to_upper(str_squish(d$award_criteria))
d$owner_org <- str_to_upper(str_squish(d$owner_org))
d$owner_org_title <- str_to_upper(str_squish(d$owner_org_title))

d$agreement_type_code[d$agreement_type_code == ''] <- '0'
d$country_of_vendor[d$country_of_vendor == ''] <- 'CA'
d$trade_agreement[d$trade_agreement == ''] <- 'XX'
d$commodity_type[d$commodity_type == ''] <- 'UNKWN'
d$solicitation_procedure[d$solicitation_procedure == ''] <- 'UNKWN'
d$limited_tendering_reason[d$limited_tendering_reason == ''] <- '0'
d$trade_agreement_exceptions[d$trade_agreement_exceptions == ''] <- '00'
d$number_of_bids[is.na(d$number_of_bids)] <- 0
d$article_6_exceptions[d$article_6_exceptions == ''] <- '0'
d$award_criteria[d$award_criteria == ''] <- '0'

d %>% filter(instrument_type == 'C') %>%
      filter(contract_value >= 0) %>% 
      filter(original_value >= 0) %>%
      group_by(procurement_id) %>%
      filter(n() == 1) %>%
      ungroup() %>%
      filter(agreement_type_code %in% c('0', 'X', 'Y', 'C', 'W', 'I', 'A', 'R', 'S', 'T', 'V', 'D', 'E', 'F', 'AB', 'AC', 'AD', 'AF', 'AG', 'AH', 'AI', 'AJ' , 'AK', 'AL', 'AM', 'AN', 'AO', 'AP', 'AQ', 'AR', 'AS', 'AT', 'AU', 'AV', 'AW', 'AX', 'AY', 'AZ', 'BA')) %>%
      filter(trade_agreement %in% c('XX', 'NA', 'GP', 'CL', 'PE', 'CO', 'PA', 'HN', 'KR', 'CA', 'EU', 'TP', 'UA', 'UK')) %>%
      filter(commodity_type %in% c('UNKWN', 'G', 'S', 'C')) %>%
      filter(solicitation_procedure %in% c('UNKWN', 'OB', 'ST', 'TC', 'AC', 'TN')) %>%
      filter(limited_tendering_reason %in% c('0', '5', '20', '21', '22', '71', '72', '74', '81', '23', '30', '31', '32', '33')) %>%
      filter(trade_agreement_exceptions %in% c('00', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14')) %>%
      filter(article_6_exceptions %in% c('0', '1', '2', '3', '4')) %>%
      filter(award_criteria %in% c('0', '1', '2', '3', '4', '5')) %>% 
      filter(str_detect(vendor_postal_code, '[:alpha:]{1}[:digit:]{1}[:alpha:]{1}') | vendor_postal_code=='') %>%
      filter(str_detect(reporting_period, '[:digit:]{4}-[:digit:]{4}-Q[:digit:]{1}')) %>%
      filter(str_detect(contract_period_start, '[:digit:]{4}-[:digit:]{2}-[:digit:]{2}')) %>%
      filter(str_detect(contract_date, '[:digit:]{4}-[:digit:]{2}-[:digit:]{2}')) %>%      
      select(vendor_name, vendor_postal_code, contract_date, economic_object_code, contract_period_start, contract_value, original_value, commodity_type, commodity_code, country_of_vendor, solicitation_procedure, limited_tendering_reason, trade_agreement_exceptions, number_of_bids, award_criteria, reporting_period, owner_org, owner_org_title) %>% 
      write.csv('contracts.cleaned.csv', row.names=FALSE)
