import pandas as pd

import plotly.express as px
import plotly.io as pio
pio.kaleido.scope.mathjax = None
pio.renderers.default = 'chromium'
pd.set_option('display.max_columns', None)

width = 800
aspect = 1.5

fuelData = pd.read_csv('/home/joerg/Teaching/BUSI4720_Dev/Visualization/fuel.csv')
contractsData = pd.read_csv('/home/joerg/Teaching/BUSI4720_Dev/Visualization/contracts.cleaned.csv', low_memory=False)

contractsData['contract_date'] = pd.to_datetime(contractsData['contract_date'])
contractsData['contract_period_start'] = pd.to_datetime(contractsData['contract_period_start'])
contractsData['economic_object_code'] = contractsData['economic_object_code'].astype('category')
contractsData['commodity_type'] = contractsData['commodity_type'].astype('category')
contractsData['country_of_vendor'] = contractsData['country_of_vendor'].astype('category')
contractsData['solicitation_procedure'] = contractsData['solicitation_procedure'].astype('category')
contractsData['limited_tendering_reason'] = contractsData['limited_tendering_reason'].astype('category')
contractsData['trade_agreement_exceptions'] = contractsData['trade_agreement_exceptions'].astype('category')
contractsData['award_criteria'] = contractsData['award_criteria'].astype('category')

def prov_from_postal(row):
    if str(row['vendor_postal_code']).startswith('A'): return 'NL'
    if str(row['vendor_postal_code']).startswith('B'): return 'NS'
    if str(row['vendor_postal_code']).startswith('C'): return 'PE'
    if str(row['vendor_postal_code']).startswith('E'): return 'NB'
    if str(row['vendor_postal_code']).startswith('G'): return 'QC'
    if str(row['vendor_postal_code']).startswith('J'): return 'QC'
    if str(row['vendor_postal_code']).startswith('L'): return 'ON'
    if str(row['vendor_postal_code']).startswith('M'): return 'ON'
    if str(row['vendor_postal_code']).startswith('N'): return 'ON'
    if str(row['vendor_postal_code']).startswith('K'): return 'ON'
    if str(row['vendor_postal_code']).startswith('P'): return 'ON'
    if str(row['vendor_postal_code']).startswith('R'): return 'MB'
    if str(row['vendor_postal_code']).startswith('S'): return 'SK'
    if str(row['vendor_postal_code']).startswith('T'): return 'AB'
    if str(row['vendor_postal_code']).startswith('V'): return 'BC'
    if str(row['vendor_postal_code']).startswith('X'): return 'NT'
    if str(row['vendor_postal_code']).startswith('Y'): return 'YK'
    return 'UNKWN'

contractsData['vendor_province'] = contractsData.apply(prov_from_postal, axis=1)
contractsData['vendor_province'] = contractsData['vendor_province'].astype('category')

def region_from_prov(row):
    if str(row['vendor_province']) in ['NL', 'NS', 'PE', 'NB']: return 'Atlantic'
    if str(row['vendor_province']) in ['YK', 'NT']: return 'Northern'
    if str(row['vendor_province']) in ['QC', 'ON']: return 'Central'
    if str(row['vendor_province']) in ['AB', 'SK', 'MB']: return 'Prairies'
    if str(row['vendor_province']) in ['BC']: return 'Western'
    if str(row['vendor_province']) in ['UNKWN']: return 'Unknown'
    return 'Unknown'

contractsData['vendor_region'] = contractsData.apply(region_from_prov, axis=1)
contractsData['vendor_region'] = contractsData['vendor_region'].astype('category')

contractsData = contractsData[(contractsData['original_value'] < contractsData['original_value'].quantile(.90)) &
                                (contractsData['contract_value'] < contractsData['contract_value'].quantile(.90))]

# ######################################
# Histogram of counts by value bins
#
#
df = contractsData

fig = px.histogram(data_frame = df,
                   x = 'contract_value',
                   histfunc = 'count',
                   title = 'Canadian Federal Procurement Contracts by Commodity Type',
                   labels = {'contract_value': 'Contract Value', 'y': 'Count'})
fig.add_vline(x = df.contract_value.mean(), line_dash='solid')
fig.add_vline(x = df.contract_value.median(), line_dash='dot')
fig.add_vline(x = df.contract_value.quantile(.05), line_dash='dash')
fig.add_vline(x = df.contract_value.quantile(.95), line_dash='dash')
fig.add_annotation(x=df.contract_value.mean(),
            y=1,
            xref='x',
            yref='paper',
            text="Mean = {:,.0f}".format(df.contract_value.mean()),
            showarrow=True,
            arrowhead=7)
fig.add_annotation(x=df.contract_value.median(),
            y=.8,
            xref='x',
            yref='paper',
            text="Median = {:,.0f}".format(df.contract_value.median()),
            showarrow=True,
            arrowhead=7)
fig.add_annotation(x=df.contract_value.quantile(.05),
            y=.9,
            xref='x',
            yref='paper',
            text="5% Quantile = {:,.0f}".format(df.contract_value.quantile(.05)),
            showarrow=True,
            arrowhead=7)
fig.add_annotation(x=df.contract_value.quantile(.95),
            y=1,
            xref='x',
            yref='paper',
            text="95% Quantile = {:,.0f}".format(df.contract_value.quantile(.95)),
            showarrow=True,
            arrowhead=7)
fig.update_layout(yaxis_title='Counts')

fig.show()
fig.write_image('px.histogram.pdf', width=width, height=width/aspect)

# ###########################################
# Area plot, two variables in one series
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2022-01-01')) & (contractsData['contract_date'] <= pd.to_datetime('2022-12-31'))]
df = df.groupby(pd.Grouper(key='contract_date', axis=0, freq='W'))['contract_value'].sum().reset_index()

fig = px.area(data_frame = df,
              x = 'contract_date',
              y = 'contract_value',
              title = 'Canadian Federal Procurement Contracts for 2022',
              labels = {'contract_value': 'Weekly Total Contract Value in C$', 'contract_date': 'Contract Dates (Calender Week)'})
fig.update_xaxes(dtick="M1", tickformat='%W', tick0='2022-01-02')
fig.update_yaxes(tickformat=',.2r')
fig.update_traces(fillcolor='red', line=dict(color='darkred'))
fig.show()
fig.write_image('px.area.pdf', width=width, height=width/aspect)

# ############################################
# Area plot, two variables in three series
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2022-01-01')) & (contractsData['contract_date'] <= pd.to_datetime('2022-12-31'))]
df = df.groupby([pd.Grouper(key='contract_date', axis=0, freq='W'),
                 pd.Grouper(key='commodity_type', axis=0)]
                )['contract_value'].sum().reset_index()
df['commodity_type'] = df['commodity_type'].astype(str)

fig = px.area(data_frame = df,
              x = 'contract_date',
              y = 'contract_value',
              color = 'commodity_type',
              title = 'Canadian Federal Procurement Contracts for 2022',
              labels = {'contract_value': 'Weekly Total Contract Value in C$', 'contract_date': 'Contract Dates (Calender Week)', 'commodity_type': 'Commodity Type'})
fig.update_xaxes(dtick="M1", tickformat='%W', tick0='2022-01-02')
fig.update_yaxes(tickformat=',.2r')
newnames = {'S':'Services', 'G': 'Goods', 'C': 'Construction'}
fig.for_each_trace(lambda t: t.update(name = newnames[t.name],
                                      legendgroup = newnames[t.name],
                                      hovertemplate = t.hovertemplate.replace(t.name, newnames[t.name])
                                     )
                  )
fig.show()
fig.write_image('px.area3.pdf', width=width, height=width/aspect)

# #######################################################
# Area plot, two variables in three series, normalized
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2022-01-01')) & (contractsData['contract_date'] <= pd.to_datetime('2022-12-31'))]
df = df.groupby([pd.Grouper(key='contract_date', axis=0, freq='W'),
                 pd.Grouper(key='commodity_type', axis=0)]
                )['contract_value'].sum().reset_index()
df['commodity_type'] = df['commodity_type'].astype(str)

fig = px.area(data_frame = df,
              x = 'contract_date',
              y = 'contract_value',
              color = 'commodity_type',
              title = 'Canadian Federal Procurement Contracts for 2022',
              groupnorm = 'percent',
              labels = {'contract_value': 'Percent of Weekly Total Contract Value in C$', 'contract_date': 'Contract Dates (Calender Week)', 'commodity_type': 'Commodity Type'})
fig.update_xaxes(dtick="M1", tickformat='%W', tick0='2022-01-02')
fig.update_yaxes(tickformat=',.2r')
newnames = {'S':'Services', 'G': 'Goods', 'C': 'Construction'}
fig.for_each_trace(lambda t: t.update(name = newnames[t.name],
                                      legendgroup = newnames[t.name],
                                      hovertemplate = t.hovertemplate.replace(t.name, newnames[t.name])
                                     )
                  )
fig.show()
fig.write_image('px.area3.normalized.pdf', width=width, height=width/aspect)

# ######################################
# Column Plot
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2017-01-01')) & (contractsData['contract_date'] <= pd.to_datetime('2022-12-31'))]
df['year'] = df['contract_date'].astype(str).str[0:4]

df = df.groupby([pd.Grouper(key='solicitation_procedure'),
                 pd.Grouper(key='commodity_type')]
                )['contract_value'].sum().reset_index()
df['solicitation_procedure'] = df['solicitation_procedure'].astype(str)

fig = px.bar(data_frame = df,
              x = 'commodity_type',
              y = 'contract_value',
              color = 'solicitation_procedure',
              title = 'Canadian Federal Procurement Contracts for 2017 to 2022',
              text_auto=True,
              # color_discrete_sequence = px.colors.colorbrewer.Paired,
              color_discrete_sequence = px.colors.qualitative.Alphabet,
              labels = {'contract_value': 'Annual Total Contract Value in C$', 'year': 'Calendar Year', 'solicitation_procedure': 'Solicitation Procedure'})
# fig.update_xaxes(dtick="M1", tickformat='%Y', tick0='2017-01-01')
fig.update_yaxes(tickformat=',.2r')
newnames = {'UNKWN': 'Unknown', 'AC':'Advanced Contract Awardd Notice', 'OB': 'Open Bidding', 'TC': 'Traditional', 'ST': 'Selective Tendering', 'TN': 'Non-competitive'}
fig.for_each_trace(lambda t: t.update(name = newnames[t.name],
                                      legendgroup = newnames[t.name],
                                      hovertemplate = t.hovertemplate.replace(t.name, newnames[t.name])
                                     )
                  )
fig.update_layout(barmode = 'group')
fig.show()
fig.write_image('px.area3.bar.grouped.pdf', width=width, height=width/aspect)
fig.update_layout(barmode = 'stack')
fig.write_image('px.area3.bar.stacked.pdf', width=width, height=width/aspect)

# ################################
# Column Plot with Patterns
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2017-01-01')) & (contractsData['contract_date'] <= pd.to_datetime('2022-12-31'))]
df['year'] = df['contract_date'].astype(str).str[0:4]

df = df.groupby([pd.Grouper(key='solicitation_procedure'),
                 pd.Grouper(key='year')]
                )['contract_value'].sum().reset_index()
df['solicitation_procedure'] = df['solicitation_procedure'].astype(str)

fig = px.bar(data_frame = df,
              x = 'year',
              y = 'contract_value',
              # color = 'solicitation_procedure',
              pattern_shape = 'solicitation_procedure',
              pattern_shape_sequence = ['.', 'x', '+', '|', '-', '/'],
              title = 'Canadian Federal Procurement Contracts for 2017 to 2022',
              text_auto=True,
              template="simple_white",
              labels = {'contract_value': 'Annual Total Contract Value in C$', 'year': 'Calendar Year', 'solicitation_procedure': 'Solicitation Procedure'})
# fig.update_xaxes(dtick="M1", tickformat='%Y', tick0='2017-01-01')
fig.update_yaxes(tickformat=',.2r')
fig.update_traces(marker=dict(color='black', line_color='black', pattern_fillmode='replace'))
newnames = {'UNKWN': 'Unknown', 'AC':'Advanced Contract Awardd Notice', 'OB': 'Open Bidding', 'TC': 'Traditional', 'ST': 'Selective Tendering', 'TN': 'Non-competitive'}
fig.for_each_trace(lambda t: t.update(name = newnames[t.name],
                                      legendgroup = newnames[t.name],
                                      hovertemplate = t.hovertemplate.replace(t.name, newnames[t.name])
                                     )
                  )
fig.show()
fig.write_image('px.area3.bar.pattern.pdf', width=width, height=width/aspect)

# ##########################
# Box Plot
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2017-01-01')) & (contractsData['contract_date'] <= pd.to_datetime('2022-12-31'))]

fig = px.box(data_frame = df,
              x = 'vendor_province',
              y = 'contract_value',
              color = 'vendor_region',
              title = 'Canadian Federal Procurement Contracts for 2017 to 2022',
              color_discrete_sequence=px.colors.qualitative.Alphabet,
              template="presentation",
              labels = {'contract_value': 'Contract Values in C$', 'vendor_province': 'Province of Vendor', 'vendor_region': 'Region of Vendor'})
# fig.update_xaxes(dtick="M1", tickformat='%Y', tick0='2017-01-01')
fig.update_yaxes(tickformat=',.2r')
fig.update_traces(width = 0.5)
fig.show()
fig.write_image('px.box.pdf', width=width, height=width/aspect)

# ###########################
# Violin Plot
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2017-01-01')) & (contractsData['contract_date'] <= pd.to_datetime('2022-12-31'))]

fig = px.violin(data_frame = df,
              x = 'vendor_province',
              y = 'contract_value',
              color = 'vendor_region',
              points = 'all',
              title = 'Canadian Federal Procurement Contracts for 2017 to 2022',
              color_discrete_sequence=px.colors.qualitative.Vivid,
              box = True,
              labels = {'contract_value': 'Contract Values in C$', 'vendor_province': 'Province of Vendor', 'vendor_region': 'Region of Vendor'})
# fig.update_xaxes(dtick="M1", tickformat='%Y', tick0='2017-01-01')
fig.update_yaxes(tickformat=',.2r')
fig.update_traces(width = 0.5)
fig.show()
fig.write_image('px.violin.pdf', width=width, height=width/aspect)

# ######################################################3
# Dot/Bubble plot
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2017-01-01')) &
                   (contractsData['contract_date'] <= pd.to_datetime('2022-12-31')) &
                   (contractsData['vendor_province'] != 'UNKWN') &
                   (contractsData['vendor_region'] == 'Atlantic')]
df['count'] = df['vendor_province'].groupby([df['vendor_province'], df['commodity_type']]).transform('count')

df['commodity_type'] = df['commodity_type'].astype(str)
df['vendor_region'] = df['vendor_region'].astype(str)
df['vendor_province'] = df['vendor_province'].astype(str)

fig = px.scatter(df,
                 x='vendor_province',
                 y='commodity_type',
                 color='vendor_province',
                 size='count',
                 size_max=100,
                 color_discrete_sequence=px.colors.colorbrewer.Paired,
                 labels={'vendor_province': 'Vendor Province', 'commodity_type': 'Commodity Type', 'count': 'Count'},
                 title='Canadian Federal Procurement Contract Counts for Atlantic Canada')
fig.update_layout(legend=dict(orientation="h", y=1.01, yanchor="bottom", x=0.5, xanchor="center"),
                  legend_title_text=None,
                  legend_bgcolor="white",
                  legend_bordercolor="black",
                  legend_borderwidth=1)
newnames = {'G': 'Goods', 'S':'Services', 'C': 'Construction', 'NS': 'Nova Scotia', 'NB': 'New Brunswick', 'NL': 'Newfoundland & Labradro', 'PE': 'PEI'}
fig.for_each_trace(lambda t: t.update(name = newnames[t.name],
                                      legendgroup = newnames[t.name],
                                      hovertemplate = t.hovertemplate.replace(t.name, newnames[t.name])
                                     )
                  )
fig.update_layout(legend= {'itemsizing': 'constant'})
fig.update_yaxes(categoryorder="array", categoryarray=['Construction', 'Goods', 'Services'])
fig.show()
fig.write_image('px.counts.pdf', width=width, height=width/aspect)


# ######################################################3
# Dot/Bubble plot
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2017-01-01')) &
                   (contractsData['contract_date'] <= pd.to_datetime('2022-12-31')) &
                   (contractsData['commodity_type'] != 'UNKWN')]

df['year'] = df['contract_date'].astype(str).str[0:4]
df['year'] = df['year'].astype(float)
df['count'] = df['vendor_province'].groupby([df['commodity_type'], df['year']]).transform('count')
df['total_value'] = df['contract_value'].groupby([df['commodity_type'], df['year']]).transform('sum')

df['commodity_type'] = df['commodity_type'].astype(str)
df['vendor_region'] = df['vendor_region'].astype(str)
df['vendor_province'] = df['vendor_province'].astype(str)

fig = px.scatter(df,
                 x='year',
                 y='total_value',
                 color='commodity_type',
                 size='count',
                 size_max=50,
                 color_discrete_sequence=px.colors.qualitative.Alphabet,
                 labels={'year': 'Year', 'commodity_type': 'Commodity Type', 'count': 'Count', 'total_value': 'Total Annual Value in C$'},
                 title='Canadian Federal Procurement Contract Counts for Atlantic Canada')
fig.update_layout(legend=dict(orientation="h", y=1.01, yanchor="bottom", x=0.5, xanchor="center"),
                  legend_title_text=None,
                  legend_bgcolor="white",
                  legend_bordercolor="black",
                  legend_borderwidth=1)
newnames = {'G': 'Goods', 'S':'Services', 'C': 'Construction', 'NS': 'Nova Scotia', 'NB': 'New Brunswick', 'NL': 'Newfoundland & Labradro', 'PE': 'PEI'}
fig.for_each_trace(lambda t: t.update(name = newnames[t.name],
                                      legendgroup = newnames[t.name],
                                      hovertemplate = t.hovertemplate.replace(t.name, newnames[t.name])
                                     )
                  )
fig.update_layout(legend= {'itemsizing': 'constant'})
fig.show()
fig.write_image('px.bubbles.pdf', width=width, height=width/aspect)

# ###############################
# Line plot
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2017-01-01')) &
                   (contractsData['contract_date'] <= pd.to_datetime('2022-12-31')) &
                   (contractsData['commodity_type'] != 'UNKWN')]

df['year'] = df['contract_date'].astype(str).str[0:4]
df['year'] = df['year'].astype(float)
# df['count'] = df['vendor_province'].groupby([df['commodity_type'], df['year']]).transform('count')

df['commodity_type'] = df['commodity_type'].astype(str)
df['vendor_region'] = df['vendor_region'].astype(str)
df['vendor_province'] = df['vendor_province'].astype(str)
df['contract_month'] = df['contract_date'].dt.to_period('M').astype(str)
grouped_df = df.groupby(['contract_month', 'commodity_type']).agg(totalcount=('contract_value', 'count'), totalvalue=('contract_value', 'sum')).reset_index()
# Create a line plot with Plotly Express
fig = px.line(grouped_df, x='contract_month', y='totalvalue', color='commodity_type', symbol='commodity_type',
              labels={'contract_month': 'Month', 'totalvalue': 'Monthly Total Contract Value', 'commodity_type': 'Commodity Type'},
              title='Monthly Canadian Federal Procurement Contract Values', markers=True)

fig.update_xaxes(type='category', categoryorder='category ascending', tickformat="%b-%y")
fig.update_traces(marker=dict(size=10))
fig.for_each_trace(lambda trace: trace.update(name='Services' if trace.name == 'S' else 'Construction' if trace.name == 'C' else 'Goods'))
fig.add_annotation(text='2021 to 2022, Contracts above C$10,000', showarrow=False, xref='paper', x=0, xanchor='left', yref='paper', y=1.05)
fig.show()
fig.write_image('px.lines.pdf', width=width, height=width/aspect)

# #######################################3
# Pie Chart
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2022-01-01')) &
                   (contractsData['contract_date'] <= pd.to_datetime('2022-12-31')) &
                   (contractsData['vendor_province'] != 'UNKWN')]

df['vendor_province'] = df['vendor_province'].astype(str)

grouped_df = df.groupby(['vendor_province']).agg(totalvalue=('contract_value', 'sum')).reset_index()

fig = px.pie(data_frame=grouped_df,
             values='totalvalue',
             names='vendor_province',
             color_discrete_sequence=px.colors.qualitative.Prism,
             labels={'vendor_province': 'Province of Vendor', 'totalvalue': '2022 Annual Total Contract Value'},
             title = '2022 Canadian Federal Contract Award Values by Province')
fig.update_traces(textinfo='percent+label', insidetextorientation='radial')
fig.show()
fig.write_image('px.pie.pdf', width=width, height=width/aspect)

# #######################################3
# Sunburst Chart
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2022-01-01')) &
                   (contractsData['contract_date'] <= pd.to_datetime('2022-12-31')) &
                   (contractsData['vendor_province'] != 'UNKWN')]

df['vendor_province'] = df['vendor_province'].astype(str)

grouped_df = df.groupby(['vendor_region', 'vendor_province']).agg(totalvalue=('contract_value', 'sum')).reset_index()

fig = px.sunburst(data_frame=grouped_df,
             path=['vendor_region', 'vendor_province'],
             values='totalvalue',
             names='vendor_province',
             color_discrete_sequence=px.colors.qualitative.Prism,
             labels={'vendor_province': 'Province of Vendor', 'totalvalue': '2022 Annual Total Contract Value'},
             title = '2022 Canadian Federal Contract Award Values by Province')
fig.update_traces(textinfo='label+percent entry', insidetextorientation='radial')
fig.show()
fig.write_image('px.sunburst.pdf', width=width, height=width/aspect)

# #######################################3
# Treemap Chart
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2022-01-01')) &
                   (contractsData['contract_date'] <= pd.to_datetime('2022-12-31')) &
                   (contractsData['vendor_province'] != 'UNKWN')]

df['vendor_province'] = df['vendor_province'].astype(str)

grouped_df = df.groupby(['vendor_region', 'vendor_province']).agg(totalvalue=('contract_value', 'sum')).reset_index()

fig = px.treemap(data_frame=grouped_df,
             path=[px.Constant('Canada'), 'vendor_region', 'vendor_province'],
             values='totalvalue',
             names='vendor_province',
             color_discrete_sequence=px.colors.qualitative.Prism,
             labels={'vendor_province': 'Province of Vendor', 'totalvalue': '2022 Annual Total Contract Value'},
             title = '2022 Canadian Federal Contract Award Values by Province')
fig.update_traces(textinfo='label+percent entry')
fig.show()
fig.write_image('px.treemap.pdf', width=width, height=width/aspect)

# #######################################3
# Donut Chart
#
#
df = contractsData[(contractsData['contract_date'] >= pd.to_datetime('2022-01-01')) &
                   (contractsData['contract_date'] <= pd.to_datetime('2022-12-31')) &
                   (contractsData['vendor_province'] != 'UNKWN')]

df['vendor_province'] = df['vendor_province'].astype(str)

grouped_df = df.groupby(['vendor_province']).agg(totalvalue=('contract_value', 'sum')).reset_index()

fig = px.pie(data_frame=grouped_df,
             values='totalvalue',
             names='vendor_province',
             color_discrete_sequence=px.colors.qualitative.Prism,
             hole = 0.4,
             labels={'vendor_province': 'Province of Vendor', 'totalvalue': '2022 Annual Total Contract Value'},
             title = '2022 Canadian Federal Contract Award Values by Province')
fig.update_traces(textinfo='percent+label')
fig.show()
fig.write_image('px.donut.pdf', width=width, height=width/aspect)

# ######################################################
# Color swatches
#
#
fig = px.colors.qualitative.swatches()
fig.write_image('px.qualitative.colors.pdf', width=width, height=width/aspect)
fig = px.colors.colorbrewer.swatches()
fig.write_image('px.colorbrewer.colors.pdf', width=width, height=width/aspect)
fig = px.colors.sequential.swatches()
fig.write_image('px.sequential.colors.pdf', width=width, height=width/aspect)

# ####################################33
# Fuel consumption data
#
fuelData = pd.read_csv('/home/joerg/Teaching/BUSI4720_Dev/Visualization/fuel.csv')

# ###############################################
# Create a trend line plot
#
#
fig = px.scatter(fuelData,
              x='Year',
              y='Range',
              color='Category',
              title='Electric Vehicle Fuel Consumption Data, Vehicle Range',
              trendline='lowess',
              trendline_options=dict(frac=0.6666))
fig.show()
newnames = {'M': 'Midsize', 'L':'Large', 'US': 'SUV (small)', 'S': 'Subcompact', 'C': 'Compact', 'WS': 'Wagon (small)', 'T': 'Two-seater', 'UL': 'SUV (large)', 'PL': 'Pickup truck'}
fig.for_each_trace(lambda t: t.update(name = newnames[t.name],
                                      legendgroup = newnames[t.name],
                                      hovertemplate = t.hovertemplate.replace(t.name, newnames[t.name])
                                     )
                  )

fig.write_image('px.trendlines.pdf', width=width, height=width/aspect)

# #############################################
# Create a trend line plot with marginals
#
df = fuelData[fuelData['Category'].isin(['US', 'UL'])]
fig = px.scatter(df,
              x='City',
              y='Hwy',
              color='Category',
              title='Electric Vehicle Fuel Consumption Data, Highway versus City Range',
              trendline='lowess',
              marginal_x='violin',
              marginal_y='rug',
              trendline_options=dict(frac=0.6666))
newnames = {'M': 'Midsize', 'L':'Large', 'US': 'SUV (small)', 'S': 'Subcompact', 'C': 'Compact', 'WS': 'Wagon (small)', 'T': 'Two-seater', 'UL': 'SUV (large)', 'PL': 'Pickup truck'}
fig.for_each_trace(lambda t: t.update(name = newnames[t.name],
                                      legendgroup = newnames[t.name],
                                      hovertemplate = t.hovertemplate.replace(t.name, newnames[t.name])
                                     )
                  )
fig.show()
fig.write_image('px.trendlines.marginals.pdf', width=width, height=width/aspect)

# #################################
# Bubble chart
#
#
df = fuelData
df = fuelData[(fuelData['Category'].isin(['S', 'M', 'L', 'C', 'UL'])) & (fuelData['Year'] > 2012)]
grouped_df = df.groupby(['Category', 'Year']).agg(range=('Range', 'mean'), meanHwy=('Hwy', 'mean'), meanCity=('City', 'mean')).reset_index()

fig = px.scatter(grouped_df,
                 x = 'meanCity',
                 y = 'meanHwy',
                 color = 'Category',
                 size = 'range',
                 animation_frame = 'Year',
                 animation_group = 'Category',
                 range_x = [0, 5],
                 range_y = [0, 5],
                 title='Electric Vehicle Fuel Consumption Data, Highway versus City Economy with Range',
                 template='presentation',
                 labels={"range" : "Range", "meanHwy": "Highway Mean Fuel Economy\n(l/100km equivalent)", "meanCity": "City Mean Fuel Economy\n(l/100km equivalent)"},
                 size_max = 50)
newnames = {'M': 'Midsize', 'L':'Large', 'US': 'SUV (small)', 'S': 'Subcompact', 'C': 'Compact', 'WS': 'Wagon (small)', 'T': 'Two-seater', 'UL': 'SUV (large)', 'PL': 'Pickup truck'}
fig.for_each_trace(lambda t: t.update(name = newnames[t.name],
                                      legendgroup = newnames[t.name],
                                      hovertemplate = t.hovertemplate.replace(t.name, newnames[t.name])
                                     )
                  )
fig.show()
fig.write_image('px.fuel.bubbles.pdf', width=width, height=width/aspect)

# #################################
# Density Heatmap
#
#
df = fuelData

fig = px.density_heatmap(df,
                 x = 'City',
                 y = 'Hwy',
                 nbinsx=20,
                 nbinsy=20,
                 color_continuous_scale=px.colors.sequential.Viridis,
                 # animation_frame = 'Year',
                 # animation_group = 'Category',
                 # range_x = [0, 5],
                 # range_y = [0, 5],
                 marginal_x="histogram",
                 marginal_y="histogram",
                 title='Electric Vehicle Fuel Consumption Data, Highway versus City Economy Model Count',
                 labels={"range" : "Range", "Hwy": "Highway Fuel Economy (l/100km equivalent)", "City": "City Fuel Economy (l/100km equivalent)", "count": "Number of Models"})
fig.show()
fig.write_image('px.heatmap.pdf', width=width, height=width/aspect)

# #################################
# Density Heatmap
#
#
df = fuelData

fig = px.density_heatmap(df,
                 x = 'City',
                 y = 'Hwy',
                 z = 'Range',
                 histfunc = 'avg',
                 nbinsx=20,
                 nbinsy=20,
                 color_continuous_scale=px.colors.sequential.Viridis,
                 # animation_frame = 'Year',
                 # animation_group = 'Category',
                 # range_x = [0, 5],
                 # range_y = [0, 5],
                 marginal_x="rug",
                 marginal_y="rug",
                 title='Electric Vehicle Fuel Consumption Data, Highway versus City Economy Mean Range',
                 labels={"range" : "Range", "Hwy": "Highway Fuel Economy (l/100km equivalent)", "City": "City Fuel Economy (l/100km equivalent)", "count": "Number of Models"})
fig.show()
fig.write_image('px.heatmap.range.pdf', width=width, height=width/aspect)
